/*
 * Copyright (C) 2010 Neil Jagdish Patel
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * version 3.0 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3.0 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Authored by Neil Jagdish Patel <njpatel@gmail.com>
 */

namespace GwibberGtk
{
  public class StreamViewTile : Gtk.HBox
  {
    private const int our_padding = 6;

    private Gtk.Alignment align;
    private Gtk.HBox   main_box;
    private Gtk.Alignment alignment;
    private Gtk.Image  icon;
    private Gtk.VBox   vbox;
    private new Gtk.Label  name;
    private Gtk.Label  time;
    private Gtk.Label  message;
    private Gwibber.Utils utils;

    public uint uid = 0;

    public StreamViewTile ()
    {
      Object (homogeneous:false, spacing:0);
    }

    construct
    {
      utils = new Gwibber.Utils ();    

      draw.connect (on_draw);

      align = new Gtk.Alignment (0.0f, 0.0f, 1.0f, 1.0f);
      align.set_padding (our_padding, our_padding, our_padding, our_padding);
      add (align);

      main_box = new Gtk.HBox (false, 12);
      align.add (main_box);

      alignment = new Gtk.Alignment (0.5f, 0.0f, 0.0f, 0.0f);
      main_box.pack_start (alignment, false, false, 0);

      icon = new Gtk.Image.from_icon_name ("stock_person", Gtk.IconSize.DIALOG);
      alignment.add (icon);

      vbox = new Gtk.VBox (false, 0);
      main_box.pack_start (vbox, true, true, 0);
      
      var hbox = new Gtk.HBox (false, 0);
      vbox.pack_start (hbox, false, false, 2);
      
      name = new Gtk.Label ("");
      name.set_markup ("<b></b>");
      name.set_alignment (0.0f, 0.5f);
      hbox.pack_start (name, true, true, 0);

      time = new Gtk.Label ("");
      time.set_markup ("<span font_weight='light' font_size='small'>24 mins</span>");
      time.set_alignment (0.0f, 1.0f);
      hbox.pack_start (time, false, false, 0);

      message = new Gtk.Label ("");
      message.selectable = true;
      message.set_single_line_mode (false);
      message.set_line_wrap (true);
      message.set_line_wrap_mode (Pango.WrapMode.WORD_CHAR);
      message.set_alignment (0.0f, 0.0f);
      vbox.pack_start (message, false, false, 2);
    }

    private bool on_draw (Cairo.Context cr)
    {
      Gtk.Allocation alloc;
      get_allocation (out alloc);

      cr.set_line_width (1.0);

      cr.set_source_rgba (1.0, 1.0, 1.0, 1.0);
      cr.move_to (alloc.x, 0.5);
      cr.line_to (alloc.x + alloc.width, 0.5);
      cr.stroke ();

      var pat = new Cairo.Pattern.linear (0, 0, 0, alloc.height);
      pat.add_color_stop_rgba (0, 1.0, 1.0, 1.0, 0.8);
      pat.add_color_stop_rgba (1.0, 1.0, 1.0, 1.0, 0.5);

      cr.rectangle (0, 0, alloc.width, alloc.height);
      cr.set_source (pat);
      cr.fill ();

      cr.set_source_rgba (0.0, 0.0, 0.0, 0.1);
      cr.move_to (alloc.x, alloc.height - 0.5);
      cr.line_to (alloc.x + alloc.width, alloc.height - 0.5);
      cr.stroke ();

      propagate_draw (align, cr);

      return true;
    }

    public void reset ()
    {
      hide ();
    }

    public void set_details (string _account,
                             string _stream,
                             string _service,
                             string _sender,
                             string _timestamp,
                             string _message,
                             string _icon_uri,
                             string _url,
                             string _html,
                             string _timestring)
    {
      name.set_markup ("<b>" + _sender + "</b>");

      icon.set_from_file(_icon_uri);

      //FIXME: Switched this off so I can see the error messages
      /*
      try 
      {
        message.set_markup (_html);
      }
      finally
      {
        debug ("MESSAGE: fallback to _message");
        message.set_markup (_message);
      }
      */
      message.set_text (_message);

      time.set_markup ("<span font_weight='light' font_size='small' color='#999'>"
                       + utils.generate_time_string ((uint)_timestamp.to_long())
                       + "</span>");
      
      queue_resize ();
      show ();
    }                        
  }
}
